@@ -88,6 +88,8 @@ class DocumentMeta(type):
             if not hasattr(b, 'get_fields'):
                 continue
             for name, field in b.get_fields().items():
+                if name == 'mongo_id' and new_id:
+                    continue
                 new_class._fields[name] = field
 
         for name, maybefield in class_dict.items():
@@ -266,11 +268,41 @@ class Document(object):
         ''' Register a subclass of this class.  Maps the subclass to the
             value of subclass.config_polymorphic_identity if available.
         '''
+        #
+        import inspect
+        for superclass in inspect.getmro(cls)[1:]:
+            if issubclass(superclass, Document):
+                superclass.add_subclass(subclass)
+
         # if not polymorphic, stop
         if hasattr(subclass, 'config_polymorphic_identity'):
             attr = subclass.config_polymorphic_identity
             cls._subclasses[attr] = subclass
 
+    @classmethod
+    def base_query(cls, exclude_subclasses=False):
+        ''' Return the base query for this kind of document. If this class is
+            not polymorphic, the query is empty. If it is polymorphic then
+            a filter is added to match only this class and its subclasses.
+
+            :param exclude_subclasses: If this is true, only match the current \
+                class. If it is false, the default, also return subclasses of \
+                this class.
+        '''
+        if not cls.config_polymorphic:
+            return {}
+        if exclude_subclasses:
+            if cls.config_polymorphic_identity:
+                return { cls.config_polymorphic : cls.config_polymorphic_identity }
+            return {}
+        keys = [key for key in cls._subclasses]
+        if cls.config_polymorphic_identity:
+            keys.append(cls.config_polymorphic_identity)
+        return {
+            cls.config_polymorphic : {
+                '$in' : keys
+            }
+        }
 
     @classmethod
     def get_subclass(cls, obj):
@@ -298,6 +330,7 @@ class Document(object):
         if sub_value is None:
             return value
         return sub_value
+
     def __eq__(self, other):
         try:
             return self.mongo_id == other.mongo_id
@@ -350,6 +383,8 @@ class Document(object):
             for key in same_extrakeys:
                 if self.__extra_fields[key] != self.__extra_fields_orig[key]:
                     update_expression['$set'][key] = self.__extra_fields[key]
+            if not update_expression['$unset']:
+                del update_expression['$unset']
 
         return update_expression
 
@@ -458,8 +493,7 @@ class Document(object):
             :param fields: A list of :class:`mongoalchemy.query.QueryField` objects \
                     for the fields to load.  If ``None`` is passed all fields  \
                     are loaded
-            '''
-
+        '''
         subclass = cls.get_subclass(obj)
         if subclass and subclass != cls:
             unwrapped = subclass.unwrap(obj, fields=fields, session=session)
@@ -575,11 +609,9 @@ class Index(object):
         self.components = []
         self.__unique = False
         self.__drop_dups = False
-
         self.__min = None
         self.__max = None
         self.__bucket_size = None
-
         self.__expire_after = None
 
     def expire(self, after):
@@ -639,6 +671,8 @@ class Index(object):
                 index?  Default to ``False``
         '''
         self.__unique = True
+        if drop_dups and pymongo.version_tuple >= (2, 7, 5): # pragma: nocover
+            raise BadIndexException("drop_dups is not supported on pymongo >= 2.7.5")
         self.__drop_dups = drop_dups
         return self
 
@@ -648,6 +682,12 @@ class Index(object):
             :param collection: the ``pymongo`` collection to ensure this index \
                     is on
         '''
+        components = []
+        for c in self.components:
+            if isinstance(c[0], Field):
+                c = (c[0].db_field, c[1])
+            components.append(c)
+
         extras = {}
         if self.__min is not None:
             extras['min'] = self.__min
@@ -657,7 +697,8 @@ class Index(object):
             extras['bucket_size'] = self.__bucket_size
         if self.__expire_after is not None:
             extras['expireAfterSeconds'] = self.__expire_after
-        collection.ensure_index(self.components, unique=self.__unique,
+
+        collection.ensure_index(components, unique=self.__unique,
             drop_dups=self.__drop_dups, **extras)
         return self
 
