@@ -34,10 +34,15 @@ from mongoalchemy.py3compat import *
 import warnings
 from uuid import uuid4
 import pymongo
+
 if hasattr(pymongo, 'mongo_client'):
     from pymongo.mongo_client import MongoClient
 else: # pragma: no cover
     from pymongo.connection import Connection as MongoClient
+
+if hasattr(pymongo, 'mongo_replica_set_client'):
+    from pymongo.mongo_replica_set_client import MongoReplicaSetClient
+
 from bson import DBRef, ObjectId
 from mongoalchemy.query import Query, QueryResult, RemoveQuery
 from mongoalchemy.document import (FieldNotRetrieved, Document,
@@ -48,6 +53,8 @@ from mongoalchemy.exceptions import (TransactionException,
                                      SessionCacheException)
 from mongoalchemy.ops import *
 
+PYMONGO_3 = pymongo.version_tuple >= (3, 0, 0)
+
 class Session(object):
 
     def __init__(self, database, tz_aware=False, timezone=None, safe=False,
@@ -90,7 +97,7 @@ class Session(object):
         return len(self.transactions) > 0
 
     @classmethod
-    def connect(self, database, timezone=None, cache_size=0, auto_ensure=True, *args, **kwds):
+    def connect(self, database, timezone=None, cache_size=0, auto_ensure=True, replica_set=None, *args, **kwds):
         ''' `connect` is a thin wrapper around __init__ which creates the
             database connection that the session will use.
 
@@ -100,6 +107,9 @@ class Session(object):
                 init function
             :param auto_ensure: Whether to implicitly call ensure_indexes on all write \
                 operations.
+            :param replica_set: The replica-set to use (as a string). If specified, \
+                :class:`pymongo.mongo_replica_set_client.MongoReplicaSetClient` is used \
+                instead of :class:`pymongo.mongo_client.MongoClient`
             :param args: arguments for :class:`pymongo.mongo_client.MongoClient`
             :param kwds: keyword arguments for :class:`pymongo.mongo_client.MongoClient`
         '''
@@ -108,7 +118,15 @@ class Session(object):
             del kwds['safe']
         if timezone is not None:
             kwds['tz_aware'] = True
-        conn = MongoClient(*args, **kwds)
+
+        if replica_set is not None:
+            if 'MongoReplicaSetClient' in globals():
+                conn = MongoReplicaSetClient(*args, replicaSet=replica_set, **kwds)
+            else: # pragma: no cover
+                conn = MongoClient(*args, replicaSet=replica_set, **kwds)
+        else:
+            conn = MongoClient(*args, **kwds)
+
         db = conn[database]
         return Session(db, timezone=timezone, safe=safe, cache_size=cache_size, auto_ensure=auto_ensure)
 
@@ -144,7 +162,8 @@ class Session(object):
         if self.transactions:
             raise TransactionException('Tried to end session with an open '
                                        'transaction')
-        self.db.connection.end_request()
+        if not PYMONGO_3:
+            self.db.connection.end_request()
 
     def insert(self, item, safe=None): # pragma: nocover
         ''' [DEPRECATED] Please use save() instead. This actually calls
@@ -204,7 +223,7 @@ class Session(object):
         if self.autoflush:
             return self.flush()
 
-    def query(self, type):
+    def query(self, type, exclude_subclasses=False):
         ''' Begin a query on the database's collection for `type`.  If `type`
             is an instance of basesting, the query will be in raw query mode
             which will not check field values or transform returned results
@@ -216,7 +235,7 @@ class Session(object):
         # read
         if isinstance(type, basestring):
             type = FreeFormDoc(type)
-        return Query(type, self)
+        return Query(type, self, exclude_subclasses=exclude_subclasses)
 
     def add_to_session(self, obj):
         obj._set_session(self)
@@ -230,7 +249,10 @@ class Session(object):
 
         kwargs = dict()
         if query._get_fields():
-            kwargs['fields'] = query._fields_expression()
+            if PYMONGO_3: # pragma: nocover
+                kwargs['projection'] = query._fields_expression()
+            else: # pragma: nocover
+                kwargs['fields'] = query._fields_expression()
 
         collection = self.db[query.type.get_collection_name()]
         cursor = collection.find(query.query, **kwargs)
@@ -415,7 +437,10 @@ class Session(object):
         if obj is not None:
             return obj
         if ref.database and self.db.name != ref.database:
-            db = self.db.connection[ref.database]
+            if PYMONGO_3: # pragma: nocover
+                db = self.db.client[ref.database]
+            else: # pragma: nocover
+                db = self.db.connection[ref.database]
         else:
             db = self.db
         value = db.dereference(ref)
